@@ -35,6 +35,8 @@ class BackupsApp(app_module.App):
 
     _version = 3
 
+    can_be_disabled = False
+
     def __init__(self) -> None:
         """Create components for the app."""
         super().__init__()
@@ -47,7 +49,7 @@ class BackupsApp(app_module.App):
             tags=manifest.tags)
         self.add(info)
 
-        menu_item = menu.Menu('menu-backups', info.name, None, info.icon,
+        menu_item = menu.Menu('menu-backups', info.name, info.icon, info.tags,
                               'backups:index', parent_url_name='system:data',
                               order=20)
         self.add(menu_item)
