@@ -12,7 +12,6 @@ import re
 
 from plinth.actions import privileged
 from plinth.app import App
-from plinth.modules import names
 from plinth.modules.email import postfix
 from plinth.modules.names.components import DomainName
 
@@ -28,13 +27,13 @@ def get_domains():
     return {'primary_domain': conf['mydomain'], 'all_domains': domains}
 
 
-def set_all_domains(primary_domain=None):
+def set_all_domains(primary_domain: str | None = None) -> None:
     """Set the primary domain and all the domains for postfix."""
     all_domains = DomainName.list_names()
     if not primary_domain:
         primary_domain = get_domains()['primary_domain']
         if primary_domain not in all_domains:
-            primary_domain = names.get_domain_name() or list(all_domains)[0]
+            primary_domain = all_domains[0]
 
     # Update configuration and don't restart daemons
     set_domains(primary_domain, list(all_domains))
